/*****************************************************************************/
/*                                                                           */
/*  NetGen kappa version                                                     */
/*  Simulator for phylogenetic networks with sequences.                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _EVENT_H
#define _EVENT_H


/***********************************************************************/
/*    An event is a member of the event queue which keeps track        */
/*    (in order) of all the future scheduled events.  Old events       */
/*    are kept on the old event list (old_event_list).                 */
/*                                                                     */
/*    Event on the old event list are in reverse order of how they     */
/*    are dealt with (ie it is a stack) and this list contains all     */
/*    kinds of old events -- processed, cancelled, etc.                */
/***********************************************************************/

struct event{
  struct event *previous_event, *next_event;
  int event_type;   // 1 = birth, 2 = death, 3 = hybridization
  int sched_status; // 1 = future,2 = processed,3 = cancelled,4 = could not exec
  double event_time;       // time for when the event is supposed to occur
  double time_created;     // time when the event was created
  long lin_id;             // id of lineage which will have this event
  long second_lin_id;      // id of second lineage if hybrid event otherwise -1
  double second_parent_time;  // time of creation second lineage (which is
                              // tracked by the parent node (if hybrid)
                              // otherwise -1
};
typedef struct event EVENTTYPE;
typedef struct event* EVENT;
typedef struct event** EVENTPTR;


/************************/
/*   the "constructor"  */
/************************/
EVENT NewEvent(int ev_type, double ev_time, long l_id, double current_time);


/************************/
/*   event management   */
/************************/
void EstablishEvent(EVENTPTR queue_ptr, LINEAGE lin, double current_time);
int FindNextEvent(LINEAGE lin, double *time_to_event);
int FindMinEvent(double b_time, double d_time, double h_time);

void InheritParentalEvent(EVENTPTR queue_ptr, long parent_lin_id, long lin_id);


// anything that modifies the queue, needs a pointer to
// to the queue (which is already a pointer!, hence a
// double pointer in effect) for everything to work
// right (otherwise the pointer value goes by value and 
// the original pointer does not get modified!)
/************************/
/*   queue management   */
/************************/
void ScheduleNewEvent(EVENTPTR queue_ptr, EVENT ev);
EVENT GetNextEvent(EVENTPTR queue_ptr);
void CancelLinEvent(EVENTPTR ev_q_ptr, EVENTPTR old_ev_list_ptr, long l_id);
void LogOldEvent(EVENTPTR old_ev_list_ptr, EVENT ev);

#endif
