/*****************************************************************************/
/*                                                                           */
/*  NetGen kappa version                                                     */
/*  Simulator for phylogenetic networks with sequences.                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _GLOBALS_H
#define _GLOBALS_H

/*********************/
/*  global constants */
/*********************/
#define MAX_NODES 2000000
#define MAX_LINEAGES MAX_NODES
#define MAX_SEQ_LENGTH 5000
#define MAX_NUM_HOM_CHROMOS 10
#define MAX_NUM_STRANDS_PER_HC 4
#define MAX_NSGW_SEEDS 10000
#define SEQ_GEN_OPTIONS_STR_LENGTH 100
#define MAX_INFILE_NAME_LENGTH 200
#define MAX_OUTGROUP_TRIES 1000

/**********************/
/*     defaults       */
/**********************/
#define PREV_UPDATE_TIME_DEFAULT -1.0
#define DEFAULT_DESIRED_HEIGHT 0.25
#define DEFAULT_DIP_RATE 1.0
#define DEFAULT_EX_RATE_VAR 1.0
#define DEFAULT_GAMMA_VALUE -1.0
#define DEFAULT_GAMMA_SCALE_VALUE -1.0
#define DEFAULT_GAMMA_SHAPE_VALUE -1
#define DEFAULT_HYB_RATE_VAR 1.0
#define DEFAULT_LAMBDA_HD -1
#define DEFAULT_NUM_HOM_CHROMOS 1
#define DEFAULT_NUM_STRANDS_PER_HC 2
#define DEFAULT_POLY_RATE 0.0
#define DEFAULT_SEQ_LENGTH 10
#define DEFAULT_SP_RATE_VAR 1.0
#define HYB_EV_DIST_THRESH_DEFAULT DBL_MAX
#define HYB_EXP_FN_THRESH_RATE_DEFAULT .20
#define HYB_HAM_THRESH_RATE_DEFAULT 1.0
#define MAX_LINE_LENGTH 200
#define MIN_OUTG_SCALAR 1.5
#define MAX_OUTG_SCALAR 2.0

/**************************************************************/
/* These are input values from the user that are needed all   */
/* over the place (and structures needed for validation test) */
/**************************************************************/
extern unsigned long seed_netgen;
extern unsigned long user_seed_netgen;

extern long strand_summed_hd;

extern int desired_height_given_yn;
extern int fake_branch_lengths_yn;
extern int gamma_shape_value;
extern int hamming_edge_report;
extern int hhd_from_root_report;
extern int hyb_parent_selection;       // how to choose second hybrid parent
                                       // 0 = min hamming distance (default) 
                                       // 1 = random (from all active)
                                       // 2 = lineage scheduled for an ev
				       //     with expected min branch length
                                       // 3 = lineage scheduled for an ev
				       //     with expected max branch length
				       // 4 = min evolutionary distance
				       // 5 = ham distance based on exp function
extern int initial_root_sequences_yn;
extern int init_sequence_length;
extern int lambda_hd;
extern int max_num_hybrids;
extern int max_outg_diff_value;
extern int max_outg_tries;
extern int min_outg_diff_value;
extern int modified_newick_report;
extern int node_listing_report;
extern int nsgw_seeds_index;
extern int nsgw_seeds_node_index;
extern int num_extant_taxa_req;
extern int num_root_hom_chromos;
extern int num_root_strands_per_hc;
extern int outgroup;
extern int outgroup_done;
extern int print_opt_reports;
extern int pid;
extern int rates_redone_yn;
extern int report_branch_hd;
extern int seq_gen_options_yn;
extern int sequences_yn;
extern int splitstree_report;
extern int temp_sequence_length;
extern int track_nsgw_seeds_yn;
extern int var_rate_yn;
extern int update_seqs_for_all_events_yn;
extern int user_seed_netgen_yn;
extern int ultrametric_yn;

extern int *nsgw_seeds_node;

extern double desired_height;
extern double dip_rate;
extern double dip_rate_orig;
extern double expected_height;
extern double ex_rate;
extern double ex_rate_var;
extern double gamma_scale_value;
extern double height_scalar;
extern double hyb_ev_dist_thresh;
extern double hyb_exp_fn_thresh_rate;
extern double hyb_ham_thresh_rate;
extern double hyb_rate;
extern double hyb_rate_orig;
extern double hyb_rate_var;
extern double hyb_rate_var_orig;
extern double poly_rate;
extern double poly_rate_orig;
extern double sp_rate;
extern double sp_rate_var;
extern double temp_perc_max_outg_diff;
extern double temp_perc_min_outg_diff;
extern double value;

extern unsigned long *nsgw_seeds;
extern char * root_sequences[MAX_NUM_HOM_CHROMOS * MAX_NUM_STRANDS_PER_HC];
extern char seq_gen_options[SEQ_GEN_OPTIONS_STR_LENGTH];
extern char infile_name[MAX_INFILE_NAME_LENGTH];

extern FILE *fp_in;
extern FILE *fp_out;


#endif
