/*****************************************************************************/
/*                                                                           */
/*  NetGen kappa version                                                     */
/*  Simulator for phylogenetic networks with sequences.                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _HYBRIDS_H
#define _HYBRIDS_H

/*************************************/
/*    Hybrid Related Functions       */
/*************************************/

void Hybrid(NETWORK nt, EVENT ev, double current_time, int hybrid_type); 


/********************************************/
/*   ways to select a second parent         */
/********************************************/
long SelectSecondParentMinHamDist(NETWORK nt, long chosen_lin, int hybrid_type);

long SelectSecondParentRandom(NETWORK nt, long chosen_lin, int hybrid_type);

long SelectSecondParentMinBL(NETWORK nt, long chosen_lin, int hybrid_type);

long SelectSecondParentMaxBL(NETWORK nt, long chosen_lin, int hybrid_type);

long SelectSecondParentMinEvDist(NETWORK nt, long chosen_lin, int hybrid_type);

long SelectSecondParentExpFn(NETWORK nt, long chosen_lin, int hybrid_type);

/**************************/
/*  hybrid sequence work  */
/**************************/

//  diploid hybrid routines
void MakeDipHybridSequences(NODE nd, char seq_gen_options[]); 

void RandomFillStrands(NODE nd, NODE parent_ptr, int hchromo_i, 
		          int start_strand, int end_strand);

//  polyploid hybrid routines
void MakePolyHybridSequences(NODE nd, char seq_gen_options[]); 

void InheritAllStrands(NODE nd, NODE parent_ptr, int start_hchromo_id);


/*****************************/
/*    hybrid topology work   */
/*****************************/
long HybridCreation(NETWORK nt, long chosen_lin, long closest_lineage, 
		    int hybrid_type, double current_time, int h_noco, 
		    int h_noca, int h_sl);

void HybridCleanUp(NETWORK nt, long chosen_lin, long closest_lineage);

void UnexecutedHybridEvent(NETWORK nt, long chosen_lin, double current_time);


/***************************************************/
/*  helper functions for making hybrids according  */
/*  to the type of hybrization being performed     */
/***************************************************/

int CheckNodeEligibility(NODE nd_1, NODE nd_2, int hybrid_type);

int CalcNumHybridHomChromos(NODE nd_1, NODE nd_2, int hybrid_type);

int CalcNumHybridStrands(NODE nd_1, NODE nd_2, int hybrid_type);

/************************************************* */
/*    Separate Debugging Routines for testing      */
/***************************************************/
void HDDebugging(NETWORK nt);
void PolyploidDebugging(NODE nd);

#endif
