/*****************************************************************************/
/*                                                                           */
/*  NetGen kappa version                                                     */
/*  Simulator for phylogenetic networks with sequences.                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include "globals.h"
#include "node.h"
#include "event.h"
#include "network.h"
#include "utilities.h"
#include "input.h"
#include "generate.h"

/******************************************************************************/
/*                                                                            */
/*                                   NetGen                                   */
/*                             (Network Generator)                            */
/*                                                                            */
/******************************************************************************/
// University of New Mexico
// M. M. Morin
 

/**************************************************************************/
/*                                 main                                   */
/**************************************************************************/

int main(const int argc, const char *argv[]) 
{

  NETWORK nt;

  pid = getpid();

  /*************************************************************************/
  /* the command line is: 'NG -i <inputfile> -o <outputfile>'              */
  /*                                   OR                                  */
  /*          'NG -i <inputfile> -o <outputfile> -r <unsigned long>'       */
  /*************************************************************************/
  if (argc < 5) 
  {
    printf("Command line is: 'NG -i <input-file> -o <output-file> [-r <unsigned long>]'\n");
    exit(0);
  };

  if (strcmp(argv[1],"-i")!=0 || strcmp(argv[3],"-o")!=0) 
  {
    printf("Command line is: 'NG -i <input-file> -o <output-file> [-r <unsigned long>]'\n");
    exit(0);
  };

  if ((argc > 6) && (strcmp(argv[5],"-r") == 0))
  {
    // get the random number seed provided on command line
    sscanf(argv[6], "%lu", &user_seed_netgen);
    user_seed_netgen_yn = 1;
  }
  else
  {
    // if not provided generate.c will set a seed from the clock
    user_seed_netgen = 0;
    user_seed_netgen_yn = 0;
  }

  strcpy(infile_name, argv[2]);

  // open the input/output files
  fp_in = fopen(argv[2], "r");
  if (fp_in == NULL) 
  {
    printf("NETGEN ERROR: Can't open the input file '%s'\n", argv[2]);
    exit(0);
  }
                                 
  fp_out = fopen(argv[4], "w");
  if (fp_out == NULL) 
  {
    printf("NETGEN ERROR: Can't open the output file '%s'\n", argv[4]);
    exit(0);
  }

  // read the input & process the info
  ReadParams();

  // make the network (and print the network from there too)
  nt = NewNetwork();
  GenerateNetwork(nt); 


  // be good & clean up
  fclose(fp_in);
  fclose(fp_out);

  CleanUpMemory(nt);

  return 1;	
}


