/*****************************************************************************/
/*                                                                           */
/*  NetGen kappa version                                                     */
/*  Simulator for phylogenetic networks with sequences.                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _NETWORK_H
#define _NETWORK_H

/**************************************************************************/
/*                      Network is our global structure                   */
/**************************************************************************/
struct network {
  NODE root;
  double clock_height;
  double ev_height;
  double avg_hd_height;
  char name[15];
  NODE NodeArray[MAX_NODES];   // has all nodes -- active or not
  long Candidates[MAX_NODES];  // array used for selection routines
  EVENT event_queue;
  EVENT old_event_list;
				    
  // note:  a lineage's/node's id should be the same as its index into NodeArray
  
  long node_id_ctr;          // tracks what the next node id should be
  long num_act_lins;  
  long num_nodes; 
  long num_edges;
  long num_leaves;
  long num_extant_taxa;
  long num_hybrids;
  long parsimony_score;
};

typedef struct network NETWORKTYPE;
typedef struct network* NETWORK;

// set-up/clean-up/utility
NETWORK NewNetwork();
void CleanUpMemory(NETWORK nt);
void InitializeCandidatePool(NETWORK nt);

// Connection Stuff
void ConnectOnlyChild(NODE parent, NODE child);
void ConnectTwoChildren(NODE parent, NODE left_child, NODE right_child);

// Node/Lineage Work (note that these functions are here because
//                    they have a network as their input)
void MakeRootNode(NETWORK nt, double current_time);
void MakeSubRootNode(NETWORK nt, double current_time);
void MakeFirstActLins(NETWORK nt, double current_time);
void MakeExtantLeaves(NETWORK nt, double current_time);
void MakeOutgroupLeaf(NETWORK nt, double current_time);

// Tree Hunting Stuff
int IsTree(NETWORK nt);
void ResetMarkers(NODE nd);
NODE FindLeftmostNode(NETWORK nt, NODE nd1, NODE nd2);
NODE TreeTraverseSearch(NODE found_node, NODE nd1, NODE nd2, NODE treenode);

double FakeBranchLength();
void UpdateAllBranchLengths(NETWORK nt, double current_time);
void UpdateAllSequences(NETWORK nt, double current_time);

double ClockHeight(NODE nd);
double EvHeight(NODE nd);
double AvgHDHeight(NETWORK nt);
void   UpdateAllHDsFromRoot(NETWORK nt);
long   CalcParsScore(NETWORK nt);
long   CalcStrandHD(NODE rt, int hchromo_i, int strand_i);
void   HDDFS(NODE parent, NODE child, int hchromo_i, int strand_i, int orig_call);


double DijkstraDist(NETWORK nt, long source_id);
void RelaxNeighbors(NETWORK nt, long node_to_process);
void RelaxRoutine(NETWORK nt, long next_to_process_id, long neighbor_id,
		  double bl);
long NextToProcess(NETWORK nt);

#endif
