/*****************************************************************************/
/*                                                                           */
/*  NetGen kappa version                                                     */
/*  Simulator for phylogenetic networks with sequences.                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _OUTPUT_H
#define _OUTPUT_H

/******************************************************************************/
/*                     the primary output function                            */
/******************************************************************************/
void NetworkReport(NETWORK nt);

/************************************/
/*   helper functions for output    */
/************************************/
void PrintRunParams(NETWORK nt);

void PrintHybridParams();

void PrintSeqParams();

void PrintNSGWSeedInfo(); 

void PrintNodes(NETWORK nt);

void PrintLeaves(NETWORK nt);

void PrintNodeSequences(NODE nd);

void PrintEdges(NETWORK nt);

void PrintParsimonyScore(NETWORK nt);

void PrintOptReports(NETWORK nt);


/*****************************************/
/*     optional reporting functions      */
/*****************************************/

/****************************************************************************/
/*                           Hamming Edge Report                            */
/*                                                                          */
/* these three functions are used to do the special hamming report on       */
/* the edges -- this report generates A LOT of output -- rarely want this!  */
/****************************************************************************/
void PrintHammingEdges(NODE nd); 

void HammingReport(NODE nd, int direction);

void PrintHamming(int hchromo_i, int strand_i, char * seqA, 
		  char * seqB, int num_levels);


/****************************************************************************/
/*                         Modified Newick Report                           */
/****************************************************************************/
void PrintNewick(NODE treenode, int bl_type);

void PrintNewickNetwork(NODE treenode, int bl_type);


/****************************************************************************/
/*                           SplitsTree Report                              */
/*                                                                          */
/* this function prints the format needed for the SplitsTree software       */
/* note that this will allow the network to be displayed in SplitsTree,     */
/* but edge lengths are not present                                         */
/****************************************************************************/
void PrintSplitsTreeFormat(NETWORK nt);

/****************************************************************************/
/*                           Node Listing Report                            */
/*                                                                          */
/* lists all nodes in sequential order with their sequences if present      */
/****************************************************************************/
void PrintAllNodes(NETWORK nt);

/****************************************************************************/
/*                    Hybrid Hamming Distance From Root                     */
/*                                                                          */
/* lists diploid hybrids and the hamming distance between them and the root */
/****************************************************************************/
void PrintHHDFromRoot(NETWORK nt);

/****************************************************************************/
/*                      Detailed Edge Listing Report                        */
/*                                                                          */
/* lists all edges (non-topologically) with detailed info including HD      */
/****************************************************************************/
void PrintDetailedEdgeListing(NETWORK nt);

/*********************************************************/
/*  these output routines are not normally called but    */
/*    are used for debugging the simulation at times     */
/*  cannot currently request these via the input file    */
/*********************************************************/
void PrintFutureEvents(FILE *fp, EVENT queue);
void PrintOldEvents(FILE *fp, EVENT list);
void PrintInternalEdgeLengths(FILE *fp, NETWORK nt);

#endif
