/*****************************************************************************/
/*                                                                           */
/*  NSGW kappa version                                                       */
/*  Wrapper code for Seq-Gen in order to request/process sequences between   */
/*  NetGen and Seq-gen.                                                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<math.h>

#include "globals.h"
#include "network.h"


/*****************************************************************************/
/*                            NewSingleStrand                                */
/*****************************************************************************/
SINGLESTRAND NewSingleStrand(int strand_id, int sl)
{
  char *temp_ptr;
  char strand_id_str[10];
  SINGLESTRAND strand;

  strand = (SINGLESTRAND)malloc(sizeof(SINGLESTRANDTYPE));

  if (strand == NULL)
  {
    fprintf(stderr, "NSGW ERROR: Could not allocate space for strand\n");
    exit(0);
  }

  temp_ptr = (char *) calloc((sl+2),sizeof(char));

  if (temp_ptr == NULL)
  {
    fprintf(stderr, "NSGW ERROR:  Cannot allocate memory for a strand's sequence\n");
    fprintf(stderr, "NSGW ERROR:  a seq length of %d was requested\n", sl);
    exit(0);
  }

  strand->sequence = temp_ptr;

  // if sequence length is long enough, put in a temp name
  if (sl > 5)
  {
    strcpy(strand->sequence, "ini");
    sprintf(strand_id_str, "%d", strand_id);
    strcat(strand->sequence, strand_id_str);
  }

  strand->id = strand_id;
  strand->ancestor_id = -1;

  return strand;
}

/*****************************************************************************/
/*                            NewHomChrom                                    */
/*****************************************************************************/
HCHROMOSOME NewHomChrom(int nsphc, int sl)
{
  int strand_i;
  HCHROMOSOME hchromo;

  hchromo = (HCHROMOSOME)malloc(sizeof(HCHROMOSOMETYPE));
 
  if (hchromo == NULL)
  {
    fprintf(stderr, "NSGW ERROR:  Could not allocate space for a chormosome\n");
    exit(0);
  }

  for (strand_i = 0; strand_i < nsphc; strand_i++)
  {
    hchromo->strands[strand_i] = NewSingleStrand(strand_i, sl);
  }

  return hchromo;
}

/*****************************************************************************/
/*                            NewNetwork                                     */
/*****************************************************************************/

NETWORK NewNetwork()
{
  char *noname = "no_name";
  int node_i;

  NETWORK nt = (NETWORK)malloc(sizeof(NETWORKTYPE));

  if (nt == NULL)
  {
    fprintf(stderr, "NSGW ERROR:  memory cannot be allocated for the network\n");
    exit(0);
  }

  nt->root = NULL;

  // initialize the node array, just to be safe
  for (node_i = 0; node_i < MAX_NODES; node_i++)
  {
    nt->ndarray[node_i] = NULL;
  }

  nt->height = 0.0;
  strcpy(nt->name, noname);
  nt->num_of_nodes = 0;
  nt->num_of_edges = 0;
  nt->num_of_leaves = 0;
  nt->num_of_extant_taxa = 0;
  nt->num_of_hybrids = 0;
  nt->num_of_arcs = 0;
  nt->outgroup_id = -1;
  nt->min_outg_diff_value = 0;
  nt->max_outg_diff_value = 0;
  nt->max_outg_tries = 0;

  return nt;
}

/*****************************************************************************/
/*                            NewNode                                        */
/*****************************************************************************/
void NewNode(NETWORK nt, long node_i, int nhc, int nsphc, int sl)
{
  int hchromo_i;
  NODE temp_node_ptr;


  temp_node_ptr = (NODE)malloc(sizeof(NODETYPE));

  if (temp_node_ptr == NULL)
  {
    fprintf(stderr, "NSGW ERROR:  memory cannot be allocated for a needed node\n");
    exit(0);
  }
  else
  {
    nt->ndarray[node_i] = temp_node_ptr;
  }

  // use the values we were given
  nt->ndarray[node_i]->num_of_hom_chromos = nhc;
  nt->ndarray[node_i]->num_of_strands_per_hc = nsphc;
  nt->ndarray[node_i]->seq_length = sl;

  nt->ndarray[node_i]->left = NULL;
  nt->ndarray[node_i]->right = NULL;
  nt->ndarray[node_i]->parent1 = NULL;
  nt->ndarray[node_i]->parent2 = NULL;
  nt->ndarray[node_i]->id = node_i;
  nt->ndarray[node_i]->leaf = 0;
  nt->ndarray[node_i]->extant = 0;
  nt->ndarray[node_i]->hybrid = 0;
  nt->ndarray[node_i]->outgroup = 0;
  nt->ndarray[node_i]->processed_yn = 0;
  nt->ndarray[node_i]->time = 0.0;
  nt->ndarray[node_i]->brlen1 = 0.0;
  nt->ndarray[node_i]->brlen2 = 0.0;

  for (hchromo_i = 0; hchromo_i < nhc; hchromo_i++)
  {
    nt->ndarray[node_i]->hom_chromos[hchromo_i] = NewHomChrom(nsphc, sl);
  }

}
