/*****************************************************************************/
/*                                                                           */
/*  NSGW kappa version                                                       */
/*  Wrapper code for Seq-Gen in order to request/process sequences between   */
/*  NetGen and Seq-gen.                                                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _NETWORK_H
#define _NETWORK_H


/******************************************************************************/
/******************************************************************************/
/*           These structures are used only by 'After' mode                   */
/******************************************************************************/
/******************************************************************************/


/*****************************************************************************/
/*            A single strand is a string of DNA at the nucleotide level     */
/*****************************************************************************/
struct single_strand
{
  char * sequence;
  int id;
  int ancestor_id;
};
typedef struct single_strand SINGLESTRANDTYPE;
typedef struct single_strand* SINGLESTRAND;


/*****************************************************************************/
/*                A homologous chromosome is a grouping of strands.          */
/*****************************************************************************/
struct homologous_chromosome
{
  SINGLESTRAND strands[MAX_NUM_STRANDS_PER_HC];
};
typedef struct homologous_chromosome HCHROMOSOMETYPE;
typedef struct homologous_chromosome* HCHROMOSOME;


/*****************************************************************************/
/*     A node refers to a node of the network.                               */
/*****************************************************************************/
struct node
{
  struct node *left, *right, *parent1, *parent2;
  long id;
  int leaf;
  int extant;
  int hybrid;
  int num_of_hom_chromos;
  int num_of_strands_per_hc;
  int outgroup;
  int seq_length;
  int processed_yn; /* flag to be used for admin work */
  double time;
  double brlen1; /* note that the branch lengths refer to the branch's */
  double brlen2; /* length from its parent, (not TO its child)         */
                 /* and these lengths are evolution, not clock based   */

  HCHROMOSOME hom_chromos[MAX_NUM_HOM_CHROMOS];
};
typedef struct node NODETYPE;
typedef struct node* NODE;

/*****************************************************************************/
/*                  A network is our primary structure                       */
/*****************************************************************************/
struct network
{
  NODE root;
  NODE ndarray[MAX_NODES];   // indexed starting at 0
  double height;               
  char name[15];              
  int num_of_nodes;           
  int num_of_edges;
  int num_of_leaves;
  int num_of_extant_taxa;
  int num_of_hybrids;
  int num_of_arcs;
  int outgroup_id;
  int min_outg_diff_value;
  int max_outg_diff_value;
  int max_outg_tries;
};
typedef struct network NETWORKTYPE;
typedef struct network* NETWORK;

// the "constructors"
SINGLESTRAND NewSingleStrand(int strand_id, int sl);
HCHROMOSOME NewHomChrom(int noca, int sl);
void NewNode(NETWORK nt, long node_i, int noco, int noca, int sl);
NETWORK NewNetwork();

#endif
