/*****************************************************************************/
/*                                                                           */
/*  NSGW kappa version                                                       */
/*  Wrapper code for Seq-Gen in order to request/process sequences between   */
/*  NetGen and Seq-gen.                                                      */
/*  Copyright (C) 2007  Monique M. Morin                                     */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<math.h>

#include "globals.h"
#include "network.h"
#include "twister.h"
#include "utilities.h"


/*****************************************************************************/
/*                         ResetProcessed                                    */
/*****************************************************************************/
void ResetProcessed(NETWORK nt)
{
  long node_i;
  
  for (node_i = 0; node_i < nt->num_of_nodes; node_i++)
  {
    nt->ndarray[node_i]->processed_yn = 0;
  }
}

/*****************************************************************************/
/*                         HammingDistance                                   */
/*****************************************************************************/
int HammingDistance(char * str_1, char * str_2)
{
  int char_i, length, count;

  // if the strings differ in length we automatically have a hamming
  // distance > 0; so add that to count right away and then we can 
  // stop comparing strings when the shortest string is exhausted 

  length = MinInt(strlen(str_1), strlen(str_2));
  count = 0;
  count = count + (abs(strlen(str_1) - strlen(str_2))); 

  // DEBUGGING
  //fprintf(stderr, "NSGW DEBUGGING:  Our length for comparison is %d (from min(%d,%d))\n", length, strlen(str_1), strlen(str_2));

  for (char_i = 0; char_i < length; char_i++)
  {
    if (str_1[char_i] != str_2[char_i])
      count++;
  }

  // DEBUGGING
  //fprintf(stderr, "NSGW DEBUG: Our two strings below have a distance of %d\n", count);
  //fprintf(stderr, "NSGW DEBUG: str1:%s, str2:%s\n", str_1, str_2);
  // END DEBUGGING


  return count;
}

/*****************************************************************************/
/*                             MinInt                                        */
/*****************************************************************************/
int MinInt(int a, int b)
{
  if (a < b)
    return a;
  else
    return b;
}


